﻿import  flash.events.Event;
import  flash.utils.Format;

/**
* EventDispatcher class. Provides a basic AS3 EventDispatcher 
* functionality.
* @author k0t0vich
*/
class  flash.events.MovieClipEventDispatcher extends MovieClip
{
	public var className:String = "EventDispatcher";
	
	private var _eventListeners:Array;
	
	/**
	 * Constructor. Creates new EventDispatcher instance.
	 */
	public function MovieClipEventDispatcher() 
	{
		super();
		_eventListeners = [];
	}
	
	/**
	 * Ссылка на родителя - если переопределяется тогда =_parent
	 */
	private var _eventParent=null;
	
	/**
	 * Adds listener Function for the specified Event.
	 * @param	event String. Event to listen to.
	 * @param	listener Function. The listening function.
	 * @param	scope Object. Listener's scope object.
	 */
	public function addEventListener(event:String,listener:Function, scope:Object):Void 
	{
		if (scope==undefined) scope = this;
		var i:Number;
		var l:Number = _eventListeners.length;
		var obj:Object;
		for (i = 0; i < l; i++)
		{
			obj = _eventListeners[i];
			if (obj.e == event && obj.f == listener && 
								obj.o == scope) return;
		}
		_eventListeners.push( { e:event, f:listener, o:scope } );
	}
	
	/**
	 * Unregisters listener Function for the specified Event.
	 * @param	event	String. Event to be unregistered from.
	 * @param	fn	Function. The listening function.
	 * @param	ob	Object. Listener's scope object.
	 */
	public function removeEventListener(event:String,listener:Function, scope:Object):Function 
	{
		if (scope==undefined) scope = this;
		var i:Number;
		var l:Number = _eventListeners.length;
		var obj:Object;
		for (i = 0; i < l; i++)
		{
			obj = _eventListeners[i];
			if (obj.e != event) continue;
			if (obj.f != listener) continue;
			if (obj.o != scope) continue;
			return _eventListeners.splice(i, 1)[0].f;
		}
		return null;
	}
	
	/**
	 * Invokes all event-listener functions with the specified Event.
	 * @param	event	Event. The event-object to be passed to the listening function.
	 */
	public function dispatchEvent(event:Event):Void 
	{
		event.currentTarget = this;
		//trace("MovieClipEventDispatcher.dispatchEvent > event : " + event);
		var i:Number = 0;
		var l:Number = _eventListeners.length;
		for (i = 0; i < l; i++)
		{
			if (event.type != _eventListeners[i].e)	continue;
			_eventListeners[i].f.apply(_eventListeners[i].o, [event.clone()]);			
		}
		dispatchBubblesEvent(event);
		delete event;
	}
	
	/**
	 * Вызов события у родителя, если событие всплываемое и родиль может диспатчить события
	 * @param	event
	 */
	public function dispatchBubblesEvent(event:Event):Void 
	{
			//trace(this+".dispatchBubblesEvent > event : " + event);
			if (event.bubbles && this["eventParent"].dispatchEvent) 
			{		
				var bubblesEvent:Event = event.clone();
				bubblesEvent.target = this["eventParent"];
				this["eventParent"].dispatchEvent(bubblesEvent);
			}
			delete event;
	}
	
	/**
	 * Очищает список подписки
	 */
	public function onUnload()
	{
		_eventListeners = [];
	}
	
	public function toString():String
	{
		return Format.formatToString(this);
	}
	
	public function get eventParent() { 
		if (_eventParent == undefined) return _parent;
		return _eventParent;
		}
	
	public function set eventParent(value):Void 
	{
		_eventParent = value;
	}
}